"""Setup the sloecode application"""
import logging

import pylons.test

from sloecode.bzr.factory import BazaarRepositoryFactory
from sloecode.config.environment import load_environment
from sloecode.model.meta import Session, metadata, Base
from sloecode.model import Person, SiteRole
from sloecode.model.site_role import USER_ADMIN, PROJECT_ADMIN


log = logging.getLogger(__name__)

def setup_app(command, conf, vars):
    """Place any commands to setup sloecode here"""
    # Don't reload the app if it was loaded under the testing environment
    if not pylons.test.pylonsapp:
        load_environment(conf.global_conf, conf.local_conf)

    # Create the tables if they don't already exist
    metadata.create_all(bind=Session.bind)

    log.info("Dropping tables")
    Base.metadata.drop_all(checkfirst=True, bind=Session.bind)
    log.info("Creating tables")
    Base.metadata.create_all(bind=Session.bind)
    log.info("Creating default admin user")
    admin = Person(login='admin', password='admin',
        name='Default Administrator')
    admin.site_roles.append(SiteRole(role=USER_ADMIN))
    admin.site_roles.append(SiteRole(role=PROJECT_ADMIN))
    Session.add(admin)

    Session.commit()
    log.info("Successfully setup")
    
    # Ensure that the directories exist for the repositories.
    fact = BazaarRepositoryFactory(
        conf.global_conf['bzr_person_root'],
        conf.global_conf['bzr_project_root'],
        conf.global_conf['bzr_deleted_person_root'],
        conf.global_conf['bzr_deleted_project_root'],
        )
    # create repo for admin user we created above:
    # TODO: this will crash if the admin user was already created, which 
    # will be the case if the database has been wiped or moved from one 
    # backend to another. We should detect this and warn about it, but 
    # probably not crash!
    fact.create_shared_repository_for_person(admin.login)
